@@ -247,7 +247,7 @@ arv_evaluator_token_free (ArvEvaluatorToken *token)
 	g_free (token);
 }
 
-void
+static void
 arv_evaluator_token_debug (ArvEvaluatorToken *token, GHashTable *variables)
 {
 	ArvValue *value;
@@ -315,7 +315,7 @@ arv_evaluator_token_is_right_parenthesis (ArvEvaluatorToken *token)
 		token->token_id == ARV_EVALUATOR_TOKEN_RIGHT_PARENTHESIS);
 }
 
-gboolean
+static gboolean
 arv_evaluator_token_compare_precedence (ArvEvaluatorToken *a, ArvEvaluatorToken *b)
 {
 	gint a_precedence;
@@ -337,8 +337,8 @@ arv_evaluator_token_compare_precedence (ArvEvaluatorToken *a, ArvEvaluatorToken
 		 a_associativity == ARV_EVALUATOR_TOKEN_ASSOCIATIVITY_RIGHT_TO_LEFT));
 }
 
-ArvEvaluatorToken *
-arv_get_next_token (char **expression, ArvEvaluatorToken *previous_token)
+static ArvEvaluatorToken *
+arv_get_next_token (char **expression, gboolean previous_token_was_operand, gboolean previous_token_was_right_parenthesis)
 {
 	ArvEvaluatorToken *token = NULL;
 	ArvEvaluatorTokenId token_id = ARV_EVALUATOR_TOKEN_UNKNOWN;
@@ -435,16 +435,14 @@ arv_get_next_token (char **expression, ArvEvaluatorToken *previous_token)
 			case ',': token_id = ARV_EVALUATOR_TOKEN_COMMA; break;
 			case '?': token_id = ARV_EVALUATOR_TOKEN_TERNARY_QUESTION_MARK; break;
 			case ':': token_id = ARV_EVALUATOR_TOKEN_TERNARY_COLON; break;
-			case '+': if (previous_token != NULL &&
-				      (arv_evaluator_token_is_operand (previous_token) ||
-				       arv_evaluator_token_is_right_parenthesis (previous_token)))
+			case '+': if (previous_token_was_operand ||
+				      previous_token_was_right_parenthesis)
 					  token_id = ARV_EVALUATOR_TOKEN_ADDITION;
 				  else
 					  token_id = ARV_EVALUATOR_TOKEN_PLUS;
 				  break;
-			case '-': if (previous_token != NULL &&
-				      (arv_evaluator_token_is_operand (previous_token) ||
-				       arv_evaluator_token_is_right_parenthesis (previous_token)))
+			case '-': if (previous_token_was_operand ||
+				      previous_token_was_right_parenthesis)
 					  token_id = ARV_EVALUATOR_TOKEN_SUBSTRACTION;
 				  else
 					  token_id = ARV_EVALUATOR_TOKEN_MINUS;
@@ -504,7 +502,7 @@ arv_get_next_token (char **expression, ArvEvaluatorToken *previous_token)
 	return token;
 }
 
-ArvEvaluatorStatus
+static ArvEvaluatorStatus
 evaluate (GSList *token_stack, GHashTable *variables, gint64 *v_int64, double *v_double)
 {
 	ArvEvaluatorToken *token;
@@ -709,18 +707,18 @@ evaluate (GSList *token_stack, GHashTable *variables, gint64 *v_int64, double *v
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_SGN:
 				if (arv_value_holds_double (&stack[index])) {
-					double value = arv_value_get_double (&stack[index]);
-					if (value < 0.0)
+					double dbl_value = arv_value_get_double (&stack[index]);
+					if (dbl_value < 0.0)
 						arv_value_set_int64 (&stack[index], -1);
-					else if (value > 0.0)
+					else if (dbl_value > 0.0)
 						arv_value_set_int64 (&stack[index], 1);
 					else
 						arv_value_set_int64 (&stack[index], 0);
 				} else {
-					gint64 value = arv_value_get_int64 (&stack[index]);
-					if (value < 0)
+					gint64 int_value = arv_value_get_int64 (&stack[index]);
+					if (int_value < 0)
 						arv_value_set_int64 (&stack[index], -1);
-					else if (value > 0)
+					else if (int_value > 0)
 						arv_value_set_int64 (&stack[index], 1);
 					else
 						arv_value_set_int64 (&stack[index], 0);
@@ -729,7 +727,7 @@ evaluate (GSList *token_stack, GHashTable *variables, gint64 *v_int64, double *v
 			case ARV_EVALUATOR_TOKEN_FUNCTION_NEG:
 				if (arv_value_holds_double (&stack[index]))
 					arv_value_set_double (&stack[index],
-							      -arv_value_get_double (&stack[index-1]));
+							      -arv_value_get_double (&stack[index]));
 				else
 					arv_value_set_int64 (&stack[index],
 							     -arv_value_get_int64 (&stack[index]));
@@ -743,10 +741,10 @@ evaluate (GSList *token_stack, GHashTable *variables, gint64 *v_int64, double *v
 			case ARV_EVALUATOR_TOKEN_FUNCTION_ABS:
 				if (arv_value_holds_double (&stack[index]))
 					arv_value_set_double (&stack[index],
-							      fabs (arv_value_get_double (&stack[index-1])));
+							      fabs (arv_value_get_double (&stack[index])));
 				else
 					arv_value_set_int64 (&stack[index],
-							     abs (arv_value_get_int64 (&stack[index])));
+							     labs (arv_value_get_int64 (&stack[index])));
 				break;
 			case ARV_EVALUATOR_TOKEN_FUNCTION_EXP:
 				arv_value_set_double (&stack[index], exp (arv_value_get_double (&stack[index])));
@@ -841,11 +839,12 @@ CLEANUP:
 
 typedef struct {
 	int count;
-	ArvEvaluatorToken *previous_token;
 	GSList *token_stack;
 	GSList *operator_stack;
 	GSList *garbage_stack;
 	gboolean in_sub_expression;
+	gboolean previous_token_was_operand;
+	gboolean previous_token_was_right_parenthesis;
 } ArvEvaluatorParserState;
 
 static ArvEvaluatorStatus
@@ -853,6 +852,7 @@ parse_to_stacks (ArvEvaluator *evaluator, char *expression, ArvEvaluatorParserSt
 {
 	ArvEvaluatorToken *token;
 	ArvEvaluatorStatus status;
+	gboolean token_found;
 
 	if (expression == NULL)
 		return ARV_EVALUATOR_STATUS_EMPTY_EXPRESSION;
@@ -861,8 +861,13 @@ parse_to_stacks (ArvEvaluator *evaluator, char *expression, ArvEvaluatorParserSt
 	/* http://en.wikipedia.org/wiki/Shunting-yard_algorithm */
 
 	do {
-		token = arv_get_next_token (&expression, state->previous_token);
+		token = arv_get_next_token (&expression, state->previous_token_was_operand, state->previous_token_was_right_parenthesis);
 		if (token != NULL) {
+			token_found = TRUE;
+
+			state->previous_token_was_operand = arv_evaluator_token_is_operand (token);
+			state->previous_token_was_right_parenthesis = arv_evaluator_token_is_right_parenthesis (token);
+
 			if (arv_evaluator_token_is_variable (token)) {
 				if (g_hash_table_lookup_extended (evaluator->priv->constants, token->data.name, NULL, NULL)) {
 					const char *constant;
@@ -871,7 +876,7 @@ parse_to_stacks (ArvEvaluator *evaluator, char *expression, ArvEvaluatorParserSt
 
 					if (constant != NULL) {
 						arv_evaluator_token_free (token);
-						token = arv_get_next_token ((char **) &constant, NULL);
+						token = arv_get_next_token ((char **) &constant, FALSE, FALSE);
 						if (token != NULL)
 							state->token_stack = g_slist_prepend (state->token_stack, token);
 					} else {
@@ -902,6 +907,7 @@ parse_to_stacks (ArvEvaluator *evaluator, char *expression, ArvEvaluatorParserSt
 						}
 
 						arv_evaluator_token_free (token);
+						token = NULL;
 					} else {
 						status = ARV_EVALUATOR_STATUS_UNKNOWN_SUB_EXPRESSION;
 						goto CLEANUP;
@@ -953,9 +959,10 @@ parse_to_stacks (ArvEvaluator *evaluator, char *expression, ArvEvaluatorParserSt
 		} else if (*expression != '\0') {
 			status = ARV_EVALUATOR_STATUS_SYNTAX_ERROR;
 			goto CLEANUP;
+		} else {
+			token_found = FALSE;
 		}
-		state->previous_token = token;
-	} while (token != NULL);
+	} while (token_found);
 
 	return ARV_EVALUATOR_STATUS_SUCCESS;
 
@@ -986,7 +993,8 @@ parse_expression (ArvEvaluator *evaluator)
 	int count;
 
 	state.count  =0;
-	state.previous_token = NULL;
+	state.previous_token_was_operand = FALSE;
+	state.previous_token_was_right_parenthesis = FALSE;
 	state.token_stack = NULL;
 	state.operator_stack = NULL;
 	state.garbage_stack = NULL;
