@@ -23,6 +23,10 @@
 #ifndef ARV_GC_PROPERTY_NODE_H
 #define ARV_GC_PROPERTY_NODE_H
 
+#if !defined (ARV_H_INSIDE) && !defined (ARAVIS_COMPILATION)
+#error "Only <arv.h> can be included directly."
+#endif
+
 #include <arvtypes.h>
 #include <arvgcnode.h>
 
@@ -91,6 +95,8 @@ struct _ArvGcPropertyNode {
 
 	ArvGcPropertyNodeType	type;
 
+	char *name;
+
 	gboolean value_data_up_to_date;
 	char *value_data;
 };
@@ -124,6 +130,7 @@ ArvGcNode * 	arv_gc_property_node_new_p_is_locked	 	(void);
 ArvGcNode * 	arv_gc_property_node_new_length			(void);
 ArvGcNode * 	arv_gc_property_node_new_p_length		(void);
 ArvGcNode * 	arv_gc_property_node_new_p_port 		(void);
+ArvGcNode * 	arv_gc_property_node_new_p_variable 		(void);
 ArvGcNode * 	arv_gc_property_node_new_formula		(void);
 ArvGcNode * 	arv_gc_property_node_new_formula_to		(void);
 ArvGcNode * 	arv_gc_property_node_new_formula_from		(void);
@@ -141,6 +148,8 @@ ArvGcNode * 	arv_gc_property_node_new_command_value		(void);
 ArvGcNode * 	arv_gc_property_node_new_p_command_value	(void);
 ArvGcNode * 	arv_gc_property_node_new_chunk_id 		(void);
 
+const char *		arv_gc_property_node_get_name		(ArvGcPropertyNode *node);
+
 const char * 		arv_gc_property_node_get_string 	(ArvGcPropertyNode *node, GError **error);
 void	 		arv_gc_property_node_set_string 	(ArvGcPropertyNode *node, const char *string, GError **error);
 gint64			arv_gc_property_node_get_int64		(ArvGcPropertyNode *node, GError **error);
