@@ -126,11 +126,13 @@ static gboolean
 _read_memory (ArvGvDeviceIOData *io_data, guint64 address, guint32 size, void *buffer, GError **error)
 {
 	ArvGvcpPacket *packet;
+	ArvGvcpPacket *ack_packet = io_data->buffer;
 	size_t packet_size;
 	size_t answer_size;
 	int count;
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
+	guint32 timeout_ms;
 
 	answer_size = arv_gvcp_packet_get_read_memory_ack_size (size);
 
@@ -148,42 +150,81 @@ _read_memory (ArvGvDeviceIOData *io_data, guint64 address, guint32 size, void *b
 						      0, &packet_size);
 
 	do {
+		GError *local_error = NULL;
+
 		io_data->packet_id = arv_gvcp_next_packet_id (io_data->packet_id);
 		arv_gvcp_packet_set_packet_id (packet, io_data->packet_id);
 
 		arv_gvcp_packet_debug (packet, ARV_DEBUG_LEVEL_LOG);
 
-		g_socket_send_to (io_data->socket, io_data->device_address,
-				  (const char *) packet, packet_size,
-				  NULL, NULL);
+		success = g_socket_send_to (io_data->socket, io_data->device_address,
+					    (const char *) packet, packet_size,
+					    NULL, &local_error) >= 0;
 
-		if (g_poll (&io_data->poll_in_event, 1, io_data->gvcp_timeout_ms) > 0) {
-			count = g_socket_receive (io_data->socket, io_data->buffer,
-						  ARV_GV_DEVICE_BUFFER_SIZE, NULL, NULL);
-			if (count >= answer_size) {
-				ArvGvcpPacket *ack_packet = io_data->buffer;
-				ArvGvcpPacketType packet_type;
-				ArvGvcpCommand command;
-				guint16 packet_id;
+		if (success) {
+			gboolean pending_ack;
+			gboolean expected_answer;
 
-				arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+			timeout_ms = io_data->gvcp_timeout_ms;
 
-				packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
-				command = arv_gvcp_packet_get_command (ack_packet);
-				packet_id = arv_gvcp_packet_get_packet_id (ack_packet);
+			do {
+				success = TRUE;
+				success = success && g_poll (&io_data->poll_in_event, 1, timeout_ms) > 0;
+				if (success)
+					count = g_socket_receive (io_data->socket, io_data->buffer,
+								  ARV_GV_DEVICE_BUFFER_SIZE, NULL, &local_error);
+				else
+					count = 0;
+				success = success && (count >= answer_size);
 
-				if (packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
-				    command == ARV_GVCP_COMMAND_READ_MEMORY_ACK &&
-				    packet_id == io_data->packet_id) {
-					memcpy (buffer, arv_gvcp_packet_get_read_memory_ack_data (ack_packet), size);
-					success = TRUE;
-				} else
-					arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_WARNING);
-			}
+				if (success) {
+					ArvGvcpPacketType packet_type;
+					ArvGvcpCommand command;
+					guint16 packet_id;
+
+					arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+
+					packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
+					command = arv_gvcp_packet_get_command (ack_packet);
+					packet_id = arv_gvcp_packet_get_packet_id (ack_packet);
+
+					if (command == ARV_GVCP_COMMAND_PENDING_ACK) {
+						pending_ack = TRUE;
+						expected_answer = FALSE;
+
+						timeout_ms = arv_gvcp_packet_get_pending_ack_timeout (ack_packet);
+
+						arv_log_device ("[GvDevice::read_memory] Pending ack timeout = %d", timeout_ms);
+					} else {
+						pending_ack = FALSE;
+						expected_answer = packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
+							command == ARV_GVCP_COMMAND_READ_MEMORY_ACK &&
+							packet_id == io_data->packet_id;
+						if (!expected_answer) {
+							arv_debug_device ("[GvDevice::read_memory] Unexpected answer (0x%04x)",
+									  packet_type);
+						}
+					}
+				} else {
+					pending_ack = FALSE;
+					expected_answer = FALSE;
+					if (local_error != NULL)
+						g_warning ("[GvDevice::read_memory] Ack reception error: %s", local_error->message);
+					g_clear_error (&local_error);
+				}
+			} while (pending_ack);
+
+			success = success && expected_answer;
+
+			if (success)
+				memcpy (buffer, arv_gvcp_packet_get_read_memory_ack_data (ack_packet), size);
+		} else {
+			if (local_error != NULL)
+				g_warning ("[GvDevice::read_memory] Command sending error: %s", local_error->message);
+			g_clear_error (&local_error);
 		}
 
 		n_retries++;
-
 	} while (!success && n_retries < io_data->gvcp_n_retries);
 
 	arv_gvcp_packet_free (packet);
@@ -207,10 +248,12 @@ static gboolean
 _write_memory (ArvGvDeviceIOData *io_data, guint64 address, guint32 size, void *buffer, GError **error)
 {
 	ArvGvcpPacket *packet;
+	ArvGvcpPacket *ack_packet = io_data->buffer;
 	size_t packet_size;
 	int count;
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
+	guint32 timeout_ms;
 
 #if GLIB_CHECK_VERSION(2,32,0)
 	g_mutex_lock (&io_data->mutex);
@@ -226,37 +269,75 @@ _write_memory (ArvGvDeviceIOData *io_data, guint64 address, guint32 size, void *
 	memcpy (arv_gvcp_packet_get_write_memory_cmd_data (packet), buffer, size);
 
 	do {
+		GError *local_error = NULL;
+
 		io_data->packet_id = arv_gvcp_next_packet_id (io_data->packet_id);
 		arv_gvcp_packet_set_packet_id (packet, io_data->packet_id);
 
 		arv_gvcp_packet_debug (packet, ARV_DEBUG_LEVEL_LOG);
 
-		g_socket_send_to (io_data->socket, io_data->device_address,
-				  (const char *) packet, packet_size,
-				  NULL, NULL);
+		success = g_socket_send_to (io_data->socket, io_data->device_address,
+					    (const char *) packet, packet_size,
+					    NULL, &local_error) >= 0;
 
-		if (g_poll (&io_data->poll_in_event, 1, io_data->gvcp_timeout_ms) > 0) {
-			count = g_socket_receive (io_data->socket, io_data->buffer,
-						  ARV_GV_DEVICE_BUFFER_SIZE, NULL, NULL);
-			if (count >= arv_gvcp_packet_get_write_memory_ack_size ()) {
-				ArvGvcpPacket *ack_packet = io_data->buffer;
-				ArvGvcpPacketType packet_type;
-				ArvGvcpCommand command;
-				guint16 packet_id;
+		if (success) {
+			gboolean pending_ack;
+			gboolean expected_answer;
 
-				arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+			timeout_ms = io_data->gvcp_timeout_ms;
 
-				packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
-				command = arv_gvcp_packet_get_command (ack_packet);
-				packet_id = arv_gvcp_packet_get_packet_id (ack_packet);
-
-				if (packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
-				    command == ARV_GVCP_COMMAND_WRITE_MEMORY_ACK &&
-				    packet_id == io_data->packet_id)
-					success = TRUE;
+			do {
+				success = TRUE;
+				success = success && g_poll (&io_data->poll_in_event, 1, io_data->gvcp_timeout_ms) > 0;
+				if (success)
+					count = g_socket_receive (io_data->socket, io_data->buffer,
+								  ARV_GV_DEVICE_BUFFER_SIZE, NULL, &local_error);
 				else
-					arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_WARNING);
-			}
+					count = 0;
+				success = success && (count >= arv_gvcp_packet_get_write_memory_ack_size ());
+
+				if (success) {
+					ArvGvcpPacketType packet_type;
+					ArvGvcpCommand command;
+					guint16 packet_id;
+
+					arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+
+					packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
+					command = arv_gvcp_packet_get_command (ack_packet);
+					packet_id = arv_gvcp_packet_get_packet_id (ack_packet);
+
+					if (command == ARV_GVCP_COMMAND_PENDING_ACK) {
+						pending_ack = TRUE;
+						expected_answer = FALSE;
+
+						timeout_ms = arv_gvcp_packet_get_pending_ack_timeout (ack_packet);
+
+						arv_log_device ("[GvDevice::write_memory] Pending ack timeout = %d", timeout_ms);
+					} else {
+						pending_ack = FALSE;
+						expected_answer = packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
+							command == ARV_GVCP_COMMAND_WRITE_MEMORY_ACK &&
+							packet_id == io_data->packet_id;
+						if (!expected_answer) {
+							arv_debug_device ("[GvDevice::write_memory] Unexpected answer (0x%04x)",
+									  packet_type);
+						}
+					}
+				} else {
+					pending_ack = FALSE;
+					expected_answer = FALSE;
+					if (local_error != NULL)
+						g_warning ("[GvDevice::write_memory] Ack reception error: %s", local_error->message);
+					g_clear_error (&local_error);
+				}
+			} while (pending_ack);
+
+			success = success && expected_answer;
+		} else {
+			if (local_error != NULL)
+				g_warning ("[GvDevice::write_memory] Command sending error: %s", local_error->message);
+			g_clear_error (&local_error);
 		}
 
 		n_retries++;
@@ -284,10 +365,12 @@ gboolean
 _read_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 *value_placeholder, GError **error)
 {
 	ArvGvcpPacket *packet;
+	ArvGvcpPacket *ack_packet = io_data->buffer;
 	size_t packet_size;
 	int count;
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
+	guint32 timeout_ms;
 
 #if GLIB_CHECK_VERSION(2,32,0)
 	g_mutex_lock (&io_data->mutex);
@@ -298,38 +381,78 @@ _read_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 *value_plac
 	packet = arv_gvcp_packet_new_read_register_cmd (address, 0, &packet_size);
 
 	do {
+		GError *local_error = NULL;
+
 		io_data->packet_id = arv_gvcp_next_packet_id (io_data->packet_id);
 		arv_gvcp_packet_set_packet_id (packet, io_data->packet_id);
 
 		arv_gvcp_packet_debug (packet, ARV_DEBUG_LEVEL_LOG);
 
-		g_socket_send_to (io_data->socket, io_data->device_address,
-				  (const char *) packet, packet_size,
-				  NULL, NULL);
+		success = g_socket_send_to (io_data->socket, io_data->device_address,
+					    (const char *) packet, packet_size,
+					    NULL, &local_error) >= 0;
 
-		if (g_poll (&io_data->poll_in_event, 1, io_data->gvcp_timeout_ms) > 0) {
-			count = g_socket_receive (io_data->socket, io_data->buffer,
-						  ARV_GV_DEVICE_BUFFER_SIZE, NULL, NULL);
-			if (count > 0) {
-				ArvGvcpPacket *ack_packet = io_data->buffer;
-				ArvGvcpPacketType packet_type;
-				ArvGvcpCommand command;
-				guint16 packet_id;
+		if (success) {
+			gboolean pending_ack;
+			gboolean expected_answer;
 
-				arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+			timeout_ms = io_data->gvcp_timeout_ms;
 
-				packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
-				command = arv_gvcp_packet_get_command (ack_packet);
-				packet_id = arv_gvcp_packet_get_packet_id (ack_packet);
+			do {
+				success = TRUE;
+				success = success && g_poll (&io_data->poll_in_event, 1, io_data->gvcp_timeout_ms) > 0;
+				if (success)
+					count = g_socket_receive (io_data->socket, io_data->buffer,
+						  ARV_GV_DEVICE_BUFFER_SIZE, NULL, &local_error);
+				else
+					count = 0;
+				success = success && (count > 0);
 
-				if (packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
-				    command == ARV_GVCP_COMMAND_READ_REGISTER_ACK &&
-				    packet_id == io_data->packet_id) {
-					*value_placeholder = arv_gvcp_packet_get_read_register_ack_value (ack_packet);
-					success = TRUE;
-				} else
-					arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_WARNING);
-			}
+				if (success) {
+					ArvGvcpPacketType packet_type;
+					ArvGvcpCommand command;
+					guint16 packet_id;
+
+					arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+
+					packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
+					command = arv_gvcp_packet_get_command (ack_packet);
+					packet_id = arv_gvcp_packet_get_packet_id (ack_packet);
+
+					if (command == ARV_GVCP_COMMAND_PENDING_ACK) {
+						pending_ack = TRUE;
+						expected_answer = FALSE;
+
+						timeout_ms = arv_gvcp_packet_get_pending_ack_timeout (ack_packet);
+
+						arv_log_device ("[GvDevice::read_register] Pending ack timeout = %d", timeout_ms);
+					} else {
+						pending_ack = FALSE;
+						expected_answer = packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
+							command == ARV_GVCP_COMMAND_READ_REGISTER_ACK &&
+							packet_id == io_data->packet_id;
+						if (!expected_answer) {
+							arv_debug_device ("[GvDevice::read_register] Unexpected answer (0x%04x)",
+									  packet_type);
+						}
+					}
+				} else {
+					pending_ack = FALSE;
+					expected_answer = FALSE;
+					if (local_error != NULL)
+						g_warning ("[GvDevice::read_register] Ack reception error: %s", local_error->message);
+					g_clear_error (&local_error);
+				}
+			} while (pending_ack);
+
+			success = success && expected_answer;
+
+			if (success)
+				*value_placeholder = arv_gvcp_packet_get_read_register_ack_value (ack_packet);
+		} else {
+			if (local_error != NULL)
+				g_warning ("[GvDevice::read_register] Command sending error: %s", local_error->message);
+			g_clear_error (&local_error);
 		}
 
 		n_retries++;
@@ -359,10 +482,12 @@ gboolean
 _write_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 value, GError **error)
 {
 	ArvGvcpPacket *packet;
+	ArvGvcpPacket *ack_packet = io_data->buffer;
 	size_t packet_size;
 	int count;
 	unsigned int n_retries = 0;
 	gboolean success = FALSE;
+	guint32 timeout_ms;
 
 #if GLIB_CHECK_VERSION(2,32,0)
 	g_mutex_lock (&io_data->mutex);
@@ -373,38 +498,74 @@ _write_register (ArvGvDeviceIOData *io_data, guint32 address, guint32 value, GEr
 	packet = arv_gvcp_packet_new_write_register_cmd (address, value, io_data->packet_id, &packet_size);
 
 	do {
+		GError *local_error = NULL;
+
 		io_data->packet_id = arv_gvcp_next_packet_id (io_data->packet_id);
 		arv_gvcp_packet_set_packet_id (packet, io_data->packet_id);
 
 		arv_gvcp_packet_debug (packet, ARV_DEBUG_LEVEL_LOG);
 
-		g_socket_send_to (io_data->socket, io_data->device_address, (const char *) packet, packet_size,
-				  NULL, NULL);
+		success = g_socket_send_to (io_data->socket, io_data->device_address, (const char *) packet, packet_size,
+					    NULL, &local_error) >= 0;
 
-		if (g_poll (&io_data->poll_in_event, 1, io_data->gvcp_timeout_ms) > 0) {
-			count = g_socket_receive (io_data->socket, io_data->buffer,
-						  ARV_GV_DEVICE_BUFFER_SIZE, NULL, NULL);
-			if (count > 0) {
-				ArvGvcpPacket *ack_packet = io_data->buffer;
-				ArvGvcpPacketType packet_type;
-				ArvGvcpCommand command;
-				guint16 packet_id;
+		if (success) {
+			gboolean pending_ack;
+			gboolean expected_answer;
 
-				arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+			timeout_ms = io_data->gvcp_timeout_ms;
 
-				packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
-				command = arv_gvcp_packet_get_command (ack_packet);
-				packet_id = arv_gvcp_packet_get_packet_id (ack_packet);
-
-				arv_log_cp ("%d, %d, %d", packet_type, command, packet_id);
-
-				if (packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
-				    command == ARV_GVCP_COMMAND_WRITE_REGISTER_ACK &&
-				    packet_id == io_data->packet_id)
-					success = TRUE;
+			do {
+				success = TRUE;
+				success = success && g_poll (&io_data->poll_in_event, 1, io_data->gvcp_timeout_ms) > 0;
+				if (success)
+					count = g_socket_receive (io_data->socket, io_data->buffer,
+								  ARV_GV_DEVICE_BUFFER_SIZE, NULL, NULL);
 				else
-					arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_WARNING);
-			}
+					count = 0;
+				success = success && (count > 0);
+
+				if (success) {
+					ArvGvcpPacketType packet_type;
+					ArvGvcpCommand command;
+					guint16 packet_id;
+
+					arv_gvcp_packet_debug (ack_packet, ARV_DEBUG_LEVEL_LOG);
+
+					packet_type = arv_gvcp_packet_get_packet_type (ack_packet);
+					command = arv_gvcp_packet_get_command (ack_packet);
+					packet_id = arv_gvcp_packet_get_packet_id (ack_packet);
+
+					if (command == ARV_GVCP_COMMAND_PENDING_ACK) {
+						pending_ack = TRUE;
+						expected_answer = FALSE;
+
+						timeout_ms = arv_gvcp_packet_get_pending_ack_timeout (ack_packet);
+
+						arv_log_device ("[GvDevice::write_register] Pending ack timeout = %d", timeout_ms);
+					} else {
+						pending_ack = FALSE;
+						expected_answer = packet_type == ARV_GVCP_PACKET_TYPE_ACK &&
+							command == ARV_GVCP_COMMAND_WRITE_REGISTER_ACK &&
+							packet_id == io_data->packet_id;
+						if (!expected_answer) {
+							arv_debug_device ("[GvDevice::write_register] Unexpected answer (0x%04x)",
+									  packet_type);
+						}
+					}
+				} else {
+					pending_ack = FALSE;
+					expected_answer = FALSE;
+					if (local_error != NULL)
+						g_warning ("[GvDevice::write_register] Ack reception error: %s", local_error->message);
+					g_clear_error (&local_error);
+				}
+			} while (pending_ack);
+
+			success = success && expected_answer;
+		} else {
+			if (local_error != NULL)
+				g_warning ("[GvDevice::write_register] Command sending error: %s", local_error->message);
+			g_clear_error (&local_error);
 		}
 
 		n_retries++;
@@ -1230,7 +1391,7 @@ arv_gv_device_new (GInetAddress *interface_address, GInetAddress *device_address
 
 	document = ARV_DOM_DOCUMENT (gv_device->priv->genicam);
 	register_description = ARV_GC_REGISTER_DESCRIPTION_NODE (arv_dom_document_get_document_element (document));
-	if (!arv_gc_register_description_node_check_schema_version (register_description, 1, 1, 0))
+	if (arv_gc_register_description_node_compare_schema_version (register_description, 1, 1, 0) < 0)
 		arv_debug_device ("[GvDevice::new] Register workaround = yes");
 
 	return ARV_DEVICE (gv_device);
