@@ -119,9 +119,6 @@ typedef struct {
 	GstElement *appsrc;
 	GstElement *transform;
 
-	guint64 timestamp_offset;
-	guint64 last_timestamp;
-
 	guint rotation;
 	gboolean flip_vertical;
 	gboolean flip_horizontal;
@@ -274,9 +271,6 @@ arv_to_gst_buffer (ArvBuffer *arv_buffer)
 						      0, buffer_size, NULL, NULL);
 	}
 
-	GST_BUFFER_DTS (buffer) = 0;
-	GST_BUFFER_DURATION (buffer) = 0;
-
 	return buffer;
 }
 
@@ -289,24 +283,8 @@ new_buffer_cb (ArvStream *stream, ArvViewer *viewer)
 	if (arv_buffer == NULL)
 		return;
 
-	if (arv_buffer_get_status (arv_buffer) == ARV_BUFFER_STATUS_SUCCESS) {
-		GstBuffer *buffer;
-		guint64 timestamp_ns;
-
-		buffer = arv_to_gst_buffer (arv_buffer);
-
-		timestamp_ns =  g_get_real_time () * 1000LL;
-
-		if (viewer->timestamp_offset == 0) {
-			viewer->timestamp_offset = timestamp_ns;
-			viewer->last_timestamp = timestamp_ns;
-		}
-
-		GST_BUFFER_DTS (buffer) = timestamp_ns - viewer->timestamp_offset;
-		GST_BUFFER_DURATION (buffer) = timestamp_ns - viewer->last_timestamp;
-
-		gst_app_src_push_buffer (GST_APP_SRC (viewer->appsrc), buffer);
-	}
+	if (arv_buffer_get_status (arv_buffer) == ARV_BUFFER_STATUS_SUCCESS)
+		gst_app_src_push_buffer (GST_APP_SRC (viewer->appsrc), arv_to_gst_buffer (arv_buffer));
 
 	if (viewer->last_buffer != NULL)
 		arv_stream_push_buffer (stream, viewer->last_buffer);
@@ -685,13 +663,22 @@ update_device_list_cb (GtkToolButton *button, ArvViewer *viewer)
 	arv_update_device_list ();
 	n_devices = arv_get_n_devices ();
 	for (i = 0; i < n_devices; i++) {
+		GString *protocol;
+
+		protocol = g_string_new (NULL);
+		g_string_append_printf (protocol, "aravis-%s-symbolic", arv_get_device_protocol (i));
+		g_string_ascii_down (protocol);
+
 		gtk_list_store_append (list_store, &iter);
 		gtk_list_store_set (list_store, &iter,
 				    0, arv_get_device_id (i),
-				    1, arv_get_device_vendor (i),
-				    2, arv_get_device_model (i),
-				    3, arv_get_device_serial_nbr (i),
+				    1, protocol->str,
+				    2, arv_get_device_vendor (i),
+				    3, arv_get_device_model (i),
+				    4, arv_get_device_serial_nbr (i),
 				    -1);
+
+		g_string_free (protocol, TRUE);
 	}
 	g_signal_handler_unblock (gtk_tree_view_get_selection (GTK_TREE_VIEW (viewer->camera_tree)), viewer->camera_selected);
 }
@@ -797,9 +784,6 @@ start_video (ArvViewer *viewer)
 			      NULL);
 	}
 
-	viewer->timestamp_offset = 0;
-	viewer->last_timestamp = 0;
-
 	arv_stream_set_emit_signals (viewer->stream, TRUE);
 	payload = arv_camera_get_payload (viewer->camera);
 	for (i = 0; i < 50; i++)
@@ -928,12 +912,12 @@ start_video (ArvViewer *viewer)
 	gst_caps_set_simple (caps,
 			     "width", G_TYPE_INT, width,
 			     "height", G_TYPE_INT, height,
-			     "framerate", GST_TYPE_FRACTION, (unsigned int ) (double) (0.5 + frame_rate), 1,
+			     "framerate", GST_TYPE_FRACTION, 0, 1,
 			     NULL);
 	gst_app_src_set_caps (GST_APP_SRC (viewer->appsrc), caps);
 	gst_caps_unref (caps);
 
-	g_object_set(G_OBJECT (viewer->appsrc), "format", GST_FORMAT_TIME, NULL);
+	g_object_set(G_OBJECT (viewer->appsrc), "format", GST_FORMAT_TIME, "is-live", TRUE, "do-timestamp", TRUE, NULL);
 
 	if (!has_gtkglsink && !has_gtksink) {
 		GstBus *bus;
